function [x,hist,t,RITZ]=arnoldi(A,v,tol,kmax,flag)

  %%% set defaults
  if nargin<5, flag=1; end
  if nargin<4, kmax=100; end
  if nargin<3, tol=1.0e-8; end
  
  %%% Set PLOTEIGS=0, if no graphical output is needed
  PLOTEIGS=1;

  
  %%% Is A a matrix?
  A_IS_MATRIX = IsMatrix(A); NoMV=0;
  
  %%% Define the matrix-vector multiplication MV.
  function c=MV(x)
    if A_IS_MATRIX, c=A*x; else, c=feval(A,x); end
    %%% Count the number of matrix vector multiplications.
    NoMV=NoMV+1;
  end
  
  %%% The value of kappa determines the numerical algorithm 
  %%% for orthonormalisation. For the options, see 'Orth'
  kappa=1;
  function [V,H]=ArnStep(V,H)
     %%% compute arnoldi's expansion vector
     w=MV(V(:,k));
     %%% orthogonalise the expansion vector
     [v,h]=Orth(V,w,kappa);
     %%% expand the set of basis vectors of the search subspace
     V=[V,v]; 
     %%% if dimension allows, expand Arnoldi's Hessenberg matrix.
     if size(v,2), H=[H;zeros(1,k-1)]; end
     H=[H,h];
  end
  
  t0=clock; 
  %%%
  [n,m]=size(v); % kmax=min(kmax,n); 
  
  if n>1000, PLOTEIGS=0; end
  if PLOTEIGS, Lambda=PlotEigs(A,n); end

  %%% The Arnoldi loop
  rho=norm(v); hist=[]; RITZ=zeros(0,2);
  %%% scale tol for relative residual reduction
  V=[v/rho]; H=zeros(1,0); gamma=1; nu=1;
  for k=1:kmax
     if rho<tol, break, end
     [V,H]=ArnStep(V,H);    %%% Expand the Arnoldi decomposition
     [Y,Ritz]=ritz(H(1:k,:),flag); %%% compute Ritz values
     y=Y(:,1); y=y/norm(y,2); %%% Scale the selected pre-Ritz vector
     %%% compute residual norm
     if k<n, rho=abs(H(k+1,k)*y(k,1)); else, rho=0; end
     if PLOTEIGS              %%% Plot the Ritz values
       PlotEigs(A,n,Ritz);
     end
     RITZ=[RITZ;[Ritz,k*ones(k,1)]]; %%% Store the Ritz values
        hist=[hist,rho]; 
   
  end
  
  x=V(:,1:size(y))*y; %%% Compute Ritz vector
  
  t=etime(clock,t0);
  
  if PLOTEIGS, plotRitz(RITZ); end  %%% Plot all 
%---------------------------------------------- 
  function LAMBDA=PlotEigs(A,n,Ritz)
     if (n>2000 &(~PLOTEIGS)), return, end

     if nargin==2, 
       %%% compute the matrix if A is a function
       if A_IS_MATRIX, AA=A; else, AA=MV(eye(n)); end
       LAMBDA=eig(AA); 
       return, 
     end

     plot(real(Lambda),imag(Lambda),'*'), hold on
     plot(real(Ritz),imag(Ritz),'ro'), hold off
     title(sprintf('Ritz values of order %d    (%d eigenvalues)',length(Ritz),length(Lambda)))
     xlabel('Convergence of the Arnoldi process')
     legend('\lambda','\theta')
     pause(0.1)
   end
%----------------------------------------------
  function plotRitz(Ritz)
    m=Ritz(end,2)+1;
    figure
    plot(real(Ritz(:,1)),Ritz(:,2),'r.')
    hold on
    m=m+0*Lambda;
    plot(real(Lambda),m,'b*')
    title('Convergence of the Ritz values')
    ylabel('The order of the Ritz values')
    xlabel('Ritz values (red .) and eigenvalues (blue *)')
  
    if ~isreal(Lambda),
      title('Convergence of the real part of the Ritz values')
      xlabel('Real part Ritz values (red .) and eigenvalues (blue *)')
      figure
      plot(imag(Ritz(:,1)),Ritz(:,2),'r.')
      hold on
      plot(imag(Lambda),m,'b*')
      strR='Convergence of the imaginary part of the Ritz values';
      title(strR)
      xlabel('Imaginary part Ritz values (red .) and eigenvalues (blue *)')
      ylabel('The order of the Ritz values')
    end
  end
%----------------------------------------------
end

%=======================================
function [Y,Ritz]=ritz(H,flag)

  [Y,D]=eig(H); Ritz=diag(D);
  %%% select pre Ritz pair
  I=mysort(Ritz,flag); Ritz=Ritz(I); Y=Y(:,I);

end
%=======================================
function I=mysort(Lambda,flag)
%%% To do: different selection criterio depending on the value of flag

       %%% select the Ritz value with largest real part
       [Lambda,I]=sort(real(Lambda));

end
%======================================

