function [theta,x,hist,t,RITZ]=lanczos(A,v,tol,kmax,flag)

  %%% set defaults
  if nargin<5, flag='lr'; end
  if nargin<4, kmax=100; end
  if nargin<3, tol=1.0e-8; end
 
  %% Store the Lanczos vectors of the eigenvectors are to be computed
  VECTOR=strcmp(lower(flag),'vectors');
  %% Alternative: apply inverse Wielandt ietration to compute
  %% the eigenvectors if the eigenvalues are known.

  %%% Set PLOTEIGS=0, if no graphical output is needed
  PLOTEIGS=1;
 
   %%% Is A a matrix?
  A_IS_MATRIX = IsMatrix(A); NoMV=0;
  
  %%% Define the matrix-vector multiplication MV.
  function c=MV(x)
    if A_IS_MATRIX, c=A*x; else, c=feval(A,x); end
    %%% Count the number of matrix vector multiplications.
    NoMV=NoMV+1;
  end
  
  t0=clock; 
  %%%
  [n,m]=size(v); 

  if n>1000, PLOTEIGS=0; end
  if PLOTEIGS, Lambda=PlotEigs(A,n); end

  rho=norm(v); v=v/rho; hist=[]; 
  %%% scale tol for relative residual reduction
 
  w=MV(v); 
  alpha=real(v'*w); 
  w=w-alpha*v;
  beta=norm(w); 
  v0=v; v=w/beta;
  if VECTOR, V=[v0,v]; end
  T=[alpha;beta]; RITZ=[alpha,1];
  
  H=zeros(1,0); gamma=1; nu=1;
  for k=2:kmax
     % if rho<tol, break, end
     %%% Compute the next Lanczos vector
     w=MV(v); 
     alpha=real(v'*w); 
     w=w-alpha*v-beta*v0;
     v0=v; beta0=beta;
     beta=norm(w);  v=w/beta;
     if VECTOR, V=[V,v]; end
     
     T(k-1:k+1,k)=[beta0;alpha;beta];
     
     %%% compute Ritz values
     [Y,D]=eig(T(1:k,:)); Ritz=diag(D);
     %%% select pre Ritz pair
     I=mysort(Ritz,flag); Ritz=Ritz(I); Y=Y(:,I);
     y=Y(:,1); y=y/norm(y,2);
     %%% compute residual norm
     rho=abs(T(k+1,k)*y(k,1));
     
     if PLOTEIGS
       PlotEigs(A,n,Ritz);
     end
     hist=[hist,rho]; RITZ=[RITZ;[Ritz,k*ones(k,1)]];
     
  end
  %%% Compute Ritz vector
  theta=Ritz(1); x=[];
  if VECTOR, x=V(:,1:size(y))*y; end

  t=etime(clock,t0);
  
  if PLOTEIGS, plotRitz(RITZ); end
  
%----------------------------------------------
  function LAMBDA=PlotEigs(A,n,Ritz)
    if nargin==2, 
      AA=feval(A,eye(n));
      tic
      LAMBDA=eig(AA);
      toc 
      return, 
    end
    plot(real(Lambda),imag(Lambda),'*'), hold on
    plot(real(Ritz),imag(Ritz),'ro'), hold off
    title(sprintf('Ritz values of order %d',length(Ritz)))
    xlabel('Convergence of the Lanczos process')
    legend('\lambda','\theta')
    pause(0.1)
  end
%----------------------------------------------
  function plotRitz(Ritz)
    m=Ritz(end,2)+1;
    figure
    plot(real(Ritz(:,1)),Ritz(:,2),'ro')
    title('Convergence of the Ritz values')
    ylabel('The order of the Ritz values')
    xlabel('Ritz values (red .)')
    hold on
    if ~isempty(Lambda)
      m=m+0*Lambda;
      plot(real(Lambda),m,'b*')
      xlabel('Ritz values (red .) and eigenvalues (blue *)')
    end
  end
%----------------------------------------------
end
%=======================================
function I=mysort(Lambda,flag)

       [Lambda,I]=sort(real(Lambda));

end

