function [V,R]=GS(A,kappa,show)
%[V,R]=GS(A,kappa)
%  Orthogonalizes using Gram-Schmidt
%
%  A is an n by k matrix
%  A=V*R with R upper k by k triangular, 
%             V is n by k orthonormal  (V'*V=I)
%
%  kappa>0, then repeated Gram-Schmidt is applied with DGKS 
%  kappa=0, classical Gram-Schmidt
%  kappa<0, modified Gram-Schmidt

global REPEATED

  if nargin<3, show=0; end
  if nargin<2, kappa=1; end
  [n,k]=size(A);
  V=zeros(n,k); R=zeros(k,k);
  for j=1:k
     %%% orthogonalize ine vector against a space spanned by V
     [v,h]=Orth(V(:,1:j-1),A(:,j),kappa);
     V(:,j)=v;
     R([1:j],j)=h;
  end

%%% display the number of repetitions 
%%% in repeated Gram-Schmidt
if show & (kappa >=0)
   plot(1:length(REPEATED),REPEATED,'+') 
end 

return
