function [x,hist,t,NoMV]=minres0(A,b,x0,tol,kmax)
% Modification MINRES that computes the Ritzvalues of the underlying
% Lanczos process.
%
%
% See   minres
%


  %%% set defaults
  if nargin<5, kmax=100; end
  if nargin<4, tol=1.0e-8; end
  if nargin<3, x0=0*b; end

%-------------------------------------------------------------------------
  %%% Is A a matrix?
  A_IS_MATRIX = IsMatrix(A); NoMV=0;
  %%% if A is not a matrix, then A is a function-handle or a function-name
  
  %%% define the matrix-vector multiplication MV
  function c=MV(x)
    if A_IS_MATRIX, c=A*x; else, c=feval(A,x); end
    %%% count the number of matrix vector multiplications
    NoMV=NoMV+1;
  end
  
%-------------------------------------------------------------------------

  t0=clock; 
  %%%
  [n,m]=size(b); 
   
  x=x0; r=b(:,1); rho=norm(r);
  %%% scale tol for relative residual reduction
  tol=rho*tol;
  %%% Update r0=r if x0 is non-zero
  if norm(x,1)>0, r=r-MV(x); rho=norm(r); end
      hist=rho;
  
  %%% initialize Lanczos
  v=r/rho; v_old=zeros(n,1); beta=0;
  BETA=[]; ALPHA=[];
  
  s=0; beta_t=0; c=-1;
  w=zeros(n,1); www=v;

  %%% scale tol for relative residual reduction
  tol=rho*tol;
  for k=1:kmax
    if abs(rho)<tol, break, end
    
    %%% Lanczos
    wv=MV(v)-beta*v_old;
    alpha=v'*wv; wv=wv-alpha*v;
    beta=norm(wv); v_old=v; v=wv/beta;
    BETA=[BETA;beta]; ALPHA=[ALPHA;alpha];
     
    %%% Update QR-decomposition of the Lanczos matrix
    l1=s*alpha - c*beta_t; l2=s*beta;
    alpha_t=-s*beta_t-c*alpha; beta_t=c*beta;
    l0 = sqrt(alpha_t*alpha_t+beta*beta); 
    c = alpha_t/l0; s = beta/l0;

    %%% update search vectors
    ww = www - l1*w; www = v - l2*w; w = ww/l0;
    %%% update approximate solution
    x =  x + (rho*c)*w; 
    %%% update residual norm
    rho =  s*rho; 
    
    hist=[hist,abs(rho)];
  end

  t=etime(clock,t0);

  ritzvalues(ALPHA,BETA);
  
end
%=============================
function theta=ritzvalues(ALPHA,BETA)

  k=length(BETA);
 
  Lanczos=spdiags([BETA,ALPHA,[0;BETA(1:k-1,1)]],-1:1,k,k);
  for j=1:50
    J=1:k;
    theta=eig(full(Lanczos(J,J)));
    theta(min(j,2))
  end

end
