function [x,hist,t,NoMV]=orthores(A,b,x0,tol,kmax)
% x is the computed solution
% hist is the sequence norm of intermediate residuals 
% t is the computing time
%
% A the name of a file that defines the action of A on a vector
% b righthandside vector
% x0 initial guess (0 if not defined)
% kmax maximal number of iterations (100 if not defined)


  %%% set defaults
  if nargin<5, kmax=100; end
  if nargin<4, tol=1.0e-8; end
  if nargin<3, x0=0*b; end
  
%-------------------------------------------------------------------------
  %%% Is A a matrix?
  A_IS_MATRIX = IsMatrix(A); NoMV=0;
  %%% if A is not a matrix, then A is a function-handle or a function-name
  
  %%% define the matrix-vector multiplication MV
  function c=MV(x)
    if A_IS_MATRIX, c=A*x; else, c=feval(A,x); end
    %%% count the number of matrix vector multiplications
    NoMV=NoMV+1;
  end
%------------------------------------------------------------------------

  t0=clock; 
  %%%
  [n,m]=size(b); 
   
  x=x0;
  r=b(:,1)-MV(x);
  mu=r'*r; 
  rho=sqrt(mu); hist=rho;
  
  %%% initialize Lanczos
  v=r; v_old=zeros(n,1); mu_old=1;
  gamma=0; x_old=zeros(n,1); 
 
  %%% scale tol for relative residual reduction
  tol=rho*tol;
  for k=1:kmax
    if rho<tol, break, end
    
    beta=gamma*mu/mu_old;
    %%% Lanczos
    wv=MV(v)-beta*v_old;
    alpha=v'*wv/mu; wv=wv-alpha*v;
    gamma=-(alpha+beta); 

    wx=v+beta*x_old; wx=wx+alpha*x; 
    v_old = v; mu_old=mu; x_old=x;  
    v = wv/gamma; mu=v'*v; x=wx/(-gamma);
    
    rho=sqrt(abs(mu)); hist=[hist,rho];
  end
  t=etime(clock,t0);

end
