close all
clear all
rand('seed',0);
rng(0,'twister')

n=1000;
%%% define matrix
P='problem1';
[A,b,Lambda]=feval(P,n);


% P='problem3';
% [A,b]=feval(P,1000)

n=size(b,1);

%%% redefine righthand side vector
x=rand(n,1);
b=feval(A,x); b=b/norm(b);

%%% Is an interval Lambda that contains all eigenvalues available?
INTERVAL=exist('Lambda');
if INTERVAL, INTERVAL=(INTERVAL & length(Lambda)>1); end

if INTERVAL
   ell=20;
   x=cos(((1:ell)-0.5)*pi/ell);
   %%% shift to interval that contains the eigenvalues 
   Lmin=min(Lambda); Lmax=max(Lambda);    %%% spectral ends
   mu=0.5*(Lmax+Lmin);                    %%% centre spectrum
   radius=0.5*(Lmax-Lmin);                %%% radius spectrum
   %%% shift x to the zeros to the interval Lambda that contains the eigenvalues 
   Mu=mu-radius*x; 
   %%% sort Mu
   I=myorder(Mu); Mu=Mu(I);
end

if 0
  NameMatrix='meier01';
  % NameMatrix='vdvorst3';
  % NameMatrix='Grond4e4';
  % NameMatrix='saad3';
  % NameMatrix='hybrid92';
  NameMatrix='tfqmr001';
  % NameMatrix='sherman1';
  % NameMatrix='pde900';
  [A,b,hbtype]=matrixhbo(NameMatrix); b=b(:,1); n=size(b,1);
  fprintf(1,'\n\n%s: %d by %d matrix of type %s\n\n',NameMatrix,n,n,hbtype)
end
if isempty(b), b=rand(n,1); end
b=b(:,1);

tol=1.0e-10;
kmax=6000;

myplot();
Methods=strvcat('Rich','LMR','ChebFix','Cheb','GCR');
Methods=strvcat('ChebFix','Cheb','GCRr','GMRES','FOM','cg','bicg');
Methods=strvcat('GCRr','GMRES','FOM','bicg','bicgstabl');
% Methods=strvcat('GMRES','bicg','bicgstab','bicgstabl','cgs');
L=4;

for k=1:size(Methods,1)
%%% Solve

method=lower(deblank(Methods(k,:)));

if strcmp(method,'rich')
%%% Polynomial solvers
  [x,hist,t,NoMV]=polynomialsolver(A,b,0*b,tol,kmax,401/2);
  myplot(A,b,x,hist,t,NoMV,method,'b');
end

if strcmp(method,'lmr')
  %%% LMR
  [x,hist,t,NoMV]=polynomialsolver(A,b,0*b,tol,kmax);
  myplot(A,b,x,hist,t,NoMV,method,'r');
end

if strcmp(method,'chebfix')
%%% Chebyshev fixed polynomial
  [x,hist,t,NoMV]=polynomialsolver(A,b,0*b,tol,kmax,Mu);
  method=sprintf('Cheb(%d)',length(Mu));
  myplot(A,b,x,hist,t,NoMV,method,'m');
end

if strcmp(method,'cheb')
  %%% Chebyshev (three-term recurrence)
  [x,hist,t,NoMV]=chebyshev(A,b,0*b,mu,radius,tol,kmax);
   myplot(A,b,x,hist,t,NoMV,method,'g');
end

str='gcr';
if strcmp(method,str)
  %%% GCR
  [x,hist,t,NoMV]=feval(str,A,b,0*b,tol,kmax);
  myplot(A,b,x,hist,t,NoMV,method,'k');
end

str='gcrr';
if strcmp(method,str)
  %%% GCRr
  [x,hist,t,NoMV]=feval(str,A,b,0*b,tol,kmax,[],7);
  myplot(A,b,x,hist,t,NoMV,method,'b');
end

str='gmres';
if strcmp(method,str)
  %%% GMRES
  [x,hist,t,NoMV]=feval(str,A,b,0*b,tol,300);
  myplot(A,b,x,hist,t,NoMV,method,'k');
end

if strcmp(method,'fom')
  %%% FOM
  [x,hist,t,NoMV]=gmres(A,b,0*b,tol,kmax,'FOM');
  myplot(A,b,x,hist,t,NoMV,method,'r');
end

str='cg';
if strcmp(method,str)
  %%% CG
  [x,hist,t,NoMV]=feval(str,A,b,0*b,tol,kmax);
  myplot(A,b,x,hist,t,NoMV,method,'m');
end

str='bicg';
if strcmp(method,str)
  %%% Bi-CG
  [x,hist,t,NoMV]=feval(str,A,b,0*b,tol,kmax);
  myplot(A,b,x,hist,t,NoMV,method,'g');
end

str='bicgstab';
if strcmp(method,str)
  %%% Bi-CGSTAB
  [x,hist,t,NoMV]=feval(str,A,b,0*b,tol,kmax);
  myplot(A,b,x,hist,t,NoMV,method,'k');
end

str='bicgstabl';
if strcmp(method,str)
  %%% Bi-CGSTAB(ell)
  [x,hist,t,NoMV]=feval(str,A,b,0*b,tol,kmax,L);
  method=sprintf('bicgstab(%d)',L);
  myplot(A,b,x,hist,t,NoMV,method,'b');
end

str='cgs';
if strcmp(method,str)
  %%% CGS
  [x,hist,t,NoMV]=feval(str,A,b,0*b,tol,kmax);
  myplot(A,b,x,hist,t,NoMV,method,'r');
end

end

myplot(A,b)

zoom on
