close all
clear all


%%% define matrix
P='problem2'; n=1000;
[A,b,Lambda]=feval(P,n);


%%% define righthand side vector
x=rand(n,1); b=feval(A,x);
b=2*rand(n,1)-1; b(1:3,1)=1000*ones(3,1);

b=b/norm(b);

%%% Is an interval Lambda that contains all eigenvalues available?
INTERVAL=exist('Lambda');
if INTERVAL, INTERVAL=(INTERVAL & length(Lambda)>1); end

if INTERVAL
   ell=20;
   x=cos(((1:ell)-0.5)*pi/ell);
   %%% shift to interval that contains the eigenvalues 
   Lmin=min(Lambda); Lmax=max(Lambda);    %%% spectral ends
   mu=0.5*(Lmax+Lmin);                    %%% centre spectrum
   radius=0.5*(Lmax-Lmin);                %%% radius spectrum
   %%% shift x to the zeros to the interval Lambda that contains the eigenvalues 
   Mu=mu-radius*x; 
   %%% sort Mu
   I=myorder(Mu); Mu=Mu(I);
end

tol=1.0e-14;
kmax=2000;
myplot();

Methods=strvcat('Rich','MR','ChebFix','Cheb');
Methods=strvcat('GCR','GCRr','GMRES','FOM');
Methods=strvcat('cg','cr','minres','symmlq','orthores','rutishauser'); 
% Methods=strvcat('cg','cr','minres','symmlq'); 

plotcolors=strvcat('b','k','r','g','m','c'); lp=size(plotcolors,1);


for k=1:size(Methods,1)
  %%% Solve  

  method=lower(deblank(Methods(k,:)));
  j=k; while (j>lp), j=j-lp; end; c=plotcolors(j,:);

  switch method

  case {'Rich','MR','ChebFix','Cheb'}
  
    if strcmp(method,'rich')
        %%% Polynomial solvers
        [x,hist,t,NoMV]=polynomialsolver(A,b,0*b,tol,kmax,401/2);
        myplot(A,b,x,hist,t,NoMV,method,'b');
    end

    if strcmp(method,'lmr')
       %%% LMR
      [x,hist,t,NoMV]=polynomialsolver(A,b,0*b,tol,kmax);
      myplot(A,b,x,hist,t,NoMV,method,'r');
    end

    if strcmp(method,'chebfix')
        %%% Chebyshev fixed polynomial
          [x,hist,t,NoMV]=polynomialsolver(A,b,0*b,tol,kmax,Mu);
          method=sprintf('Cheb(%d)',length(Mu));
          myplot(A,b,x,hist,t,NoMV,method,'m');
    end

    if strcmp(method,'cheb')
      %%% Chebyshev (three-term recurrence)
      [x,hist,t,NoMV]=chebyshev(A,b,0*b,mu,radius,tol,kmax);
       myplot(A,b,x,hist,t,NoMV,method,'g');
    end


  case {'gcr','gmres'}
    [x,hist,t,NoMV]=feval(method,A,b,0*b,tol,kmax);
    myplot(A,b,x,hist,t,NoMV,method,'k');


  case {'GCRr','GMRES'}
    str='gcrr';
    if strcmp(method,str)
      %%% GCRr
       [x,hist,t,NoMV]=feval(str,A,b,0*b,tol,kmax,[],7);
       myplot(A,b,x,hist,t,NoMV,method,'b');
    end

    if strcmp(method,'fom')
      %%% FOM
      [x,hist,t,NoMV]=gmres(A,b,0*b,tol,kmax,'FOM');
       myplot(A,b,x,hist,t,NoMV,method,'r');
    end

  case {'cg','cr','minres','minres0','symmlq','orthores','rutishauser'}
    [x,hist,t,NoMV]=feval(method,A,b,0*b,tol,kmax);
    myplot(A,b,x,hist,t,NoMV,method);

  end

end

myplot(A,b)

zoom on

